{
   imguwTGA, Truevision Targa image writer for dImage
   Copyright (C) 2009. Dejan Boras

   This file is part of dImage.

   dImage is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   dImage is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with dImage.  If not, see <http://www.gnu.org/licenses/>.

   Started On:    26.06.2009.
   Last Update:   12.03.2011.
}

{TODO: Add support for indexed image types and RLE compression}

{$MODE OBJFPC}{$H+}{$I-}
UNIT imguwTGA;

INTERFACE

   USES uError, uImage, uFileHandlers;

IMPLEMENTATION

   USES imguTGAStuff;

VAR
   tgaExt: fhTExtension;
   tgaWriter: fhTHandler;

{this routine is the purpose of this unit, it is the targa image writer}
procedure dTGAWrite(data: pointer; const fhdata: fhTData);
var
   ld: imgPLoaderData;
   Header: tgaTHeader;
   imgSpec: tgaTImageSpec;

   XFileFooter: tgaTFooter;

   imgP: imgPImage;

{write an uncompressed true color image}
procedure writeUncompressedTrueColor();
begin
   imglBlockWrite(ld^, imgP^.Image^, imgP^.Size);
   if(imgError <> 0) then exit;
end;

begin {dTGAWrite}
   ld := data; imgP := ld^.img;

   {check if the image is of a supported format}
   if(imgP^.PixF <> PIXF_RGB) and (imgP^.PixF <> PIXF_RGBA) and
   (imgP^.PixF <> PIXF_BGR) and (imgP^.PixF <> PIXF_BGRA) then exit;

   {image header}
   Header.lengthID := 0;
   Header.typeColorMap := 0;
   Header.typeImage := TGA_UNCOMPRESSED_TRUE_COLOR;

   Header.cmSpec.cmEntrySize := 0;
   Header.cmSpec.cmLength := 0;
   Header.cmSpec.FirstEntry := 0;

   {image specifications}
   imgSpec.PixDepth := imgPIXFDepth(imgP^.PixF);
   imgSpec.xOrigin := 0;
   imgSpec.yOrigin := 0;
   imgSpec.Width := imgP^.Width;
   imgSpec.Height := imgP^.Height;

   imgSpec.imgDescriptor := 0;

   {setup the origin}
   if(imgP^.Origin and imgcBT_ORIGIN_HORIZONTAL > 0) then
      imgSpec.imgDescriptor := imgSpec.imgDescriptor or TGA_ORIGIN_HORIZONTAL;
   if(imgP^.Origin and imgcBT_ORIGIN_VERTICAL = 0) then
      imgSpec.imgDescriptor := imgSpec.imgDescriptor or TGA_ORIGIN_VERTICAL;

   {write the header}
   imglBlockWrite(ld^, Header, SizeOf(Header));
   if(imgError <> 0) then exit;

   {write the image speciciation}
   imglBlockWrite(ld^, imgSpec, SizeOf(imgSpec));
   if(imgError <> 0) then exit;

   {NOTE: Write here the image ID}
   {TODO: Write here the image palette}

   {write the image down}
   writeUncompressedTrueColor();
   if(imgError <> 0) then exit;

   {prepare the footer}
   XFileFooter.offsetDev := 0;
   XFileFooter.offsetExt := 0;
   XFileFooter.Signature := tgacFooterSignature;
   XFileFooter.charDot := '.';
   XFileFooter.charNull := #0;

   {write the footer}
   imglBlockWrite(ld^, XFileFooter, SizeOf(XFileFooter));
   if(imgError <> 0) then exit;

   {success}
end;

BEGIN
   {setup the extension}
   tgaExt.ext := '.tga';
   tgaExt.Handler := @tgaWriter;

   {setup the writer}
   tgaWriter.Name := 'TGA';
   tgaWriter.Handle := @dTGAWrite;

   {register the extension and the writer}
   fhRegisterExt(@tgaExt, imgWritersInfo);
   fhRegisterHandler(@tgaWriter, imgWritersInfo);
END.
